/*
 * Written by Ed Hong UWT Feb. 19, 2003.
 * Modified by Donald Chinn May 14, 2003.
  * Modified by Ed Hong, Nov. 22, 2004.
 *  Modified by Ed Hong Feb. 28, 2005.
 */

/**
 * Class that represents an edge in a graph.
 * An object (usually some sort of data) can be associated with the edge. If edges
 *   do not need to be named, than this object can be used to represent the weight
 *   of the edge. This is stored in the elementdat instance variable.
 * A label (also represented by an object) can also be
 *   associated with an edge.  This allows you to store anything extra
 *   at an edge. For example, you could use this to mark a vertex as being visited
 *   in some graph traversal.
 *
 *
 * @author edhong
 * @version 0.0
 */
public class Edge {
    /** the first endpoint of the edge */
    Vertex v1;

    /** the second endpoint of the edge */
    Vertex v2;

    private Object elementdat;  // an object associated with this edge (often a Double).
    private Object visitLabel;  // a label associated with this edge, used in DFS traversals.

    /**
     * Constructor that allows an object to be associated
     * with the edge.
     * @param v     the first endpoint of this edge
     * @param w     the second endpoint of this edge
     * @param o     an object to be associated with this edge
		            for weighted graphs with unnamed edges, this is a Double
		            representing the edge weight.
     */
    public Edge (Vertex v, Vertex w, Object o) {
        this.elementdat = o;
        this.v1 = v;
        this.v2 = w;
    }

    /**
     * Return the object associated with this edge. In our implementation for weighted graphs
     * where edges do not have names, this is a Double.
     * @return  the object of this edge
     */
    public Object element(){
        return this.elementdat;
    }

    /**
     * Return the label associated with this edge.
     * @return  the label of this edge
     */
    public Object getLabel() {
        return visitLabel;
    }

    /**
     * Set the label associated with this edge.
     * @param o  the label of this edge
     */
    public void setLabel(Object o) {
        visitLabel = o;
    }

	/**
     * Creates a string representation of this edge.
     * @returns  string representing this edge
     */
    public String toString() {
	   String returnval;
	   returnval = "(" + v1 +"," + v2 + ")";
	   if (elementdat != null) {
	      returnval = returnval + ":" + elementdat;
	      if (visitLabel != null)
	         returnval = returnval + "," + visitLabel;
		  returnval = returnval + ":";
	   }
	   return returnval;
	}

}
