/**
 * Written by Ed Hong UWT Feb. 19, 2003.
 * Modified by Donald Chinn May 14, 2003.
 * Modified by Ed Hong, Nov. 22, 2004.
 * Modified by Ed Hong, Feb. 28, 2005.
 */

import java.io.*;
import java.util.*;
import java.text.*;

/**
 * A class that can read a graph (in a specific format) from a file.
 *
 * @author edhong
 * @version 0.0
 */
public class GraphInput {

    /**
     * Load graph data from a text file via user interaction.
     * This method asks the user for a directory and path name.
     * It returns a hash table of (String, Vertex) pairs.
     * newgraph needs to already be initialized.
     * @param newgraph  a simple graph
     * @returns a hash table of (String, Vertex) pairs
     */
    public static HashMap LoadSimpleGraph(SimpleGraph newgraph){
        System.out.print("Please enter the full path and file name for the input data: ");
        String userinput;
		Scanner keyboardscanner = new Scanner(System.in);
        userinput = keyboardscanner.nextLine();
		HashMap m;
		m = LoadSimpleGraph(newgraph, userinput);
        return m;
    }

    /**
     * Load graph data from a text file.
     * The format of the file is:
     * Each line of the file contains 3 tokens, where the first two are strings
     * representing vertex names and the third is an edge weight (a Double).
     * Each line represents one edge.
     * Each vertex is identified by a String name stored in the objectID.
     * A Double is stored with each Edge indicating the weight of the Edge.
     *
     * This method returns a hash table of (String, Vertex) pairs.
     *
     * @param newgraph  a graph to add edges to. newgraph should already be initialized
     * @param pathandfilename  the name of the file, including full path.
     * @returns  a hash table of (String, Vertex) pairs
     */
    public static HashMap LoadSimpleGraph(SimpleGraph newgraph, String pathandfilename) {
		InputStream instream;
		Scanner filescanner;
		try {
	 	   instream = new FileInputStream(pathandfilename);
	 	   filescanner = new Scanner(instream);
	    }
	    catch (java.io.FileNotFoundException notFound){
		   System.out.println("File " +pathandfilename+ " not found!");
		   return null;
		}
   	 	System.out.println("Opened " + pathandfilename + " for input.");
	    String  line = filescanner.nextLine(); // get first line
        StringTokenizer sTok;
        int n, linenum = 0;
        HashMap table = new HashMap();
        SimpleGraph sg = newgraph;

        while (line != null) {
            linenum++;
            sTok = new StringTokenizer(line);
            n = sTok.countTokens();
            if (n==3) {
                Double edgedata;
                Vertex v1, v2;
                String v1name, v2name;

                v1name = sTok.nextToken();
                v2name = sTok.nextToken();
                edgedata = new Double(Double.parseDouble(sTok.nextToken()));
                v1 = (Vertex) table.get(v1name);
                v2 = (Vertex) table.get(v2name);
                if (v1 == null) {
//                      System.out.println("New vertex " + v1name);
                        v1 = sg.insertVertex(v1name);
                        table.put(v1name, v1);
                }
                if (v2 == null) {
//                      System.out.println("New vertex " + v2name);
                    v2 = sg.insertVertex(v2name);
                    table.put(v2name, v2);
                }
//              System.out.println("Inserting edge (" + v1name + "," + v2name + ")" + edgedata);
                sg.insertEdge(v1,v2,edgedata);
            }
            else {
                System.err.println("Error:invalid number of tokens found on line " +linenum+ "!");
                return null;
            }
            if (filescanner.hasNextLine()) {
			    line = filescanner.nextLine();
			}
			else line = null;
        }

        System.out.println("Successfully loaded "+ linenum + " lines. ");
        return table;
    }


    /**
     * Code to test the methods of this class.
     */
    public static void main (String args[]) throws java.io.FileNotFoundException{
          SimpleGraph G;
          G = new SimpleGraph();
		  if (args.length ==0) LoadSimpleGraph(G);
		  else LoadSimpleGraph(G, args[0]);
		  System.out.println(G);
    }
}