/* This is a simple extension of BinaryHeapArrayPQ to make
   it be a maximizing heap instead of a minimizing heap.
 * Created by Ed Hong, 2/17/05
 *
 * The relevant implemented calls are
 	public void insertItem(double k, Object o);  -> associate key k with object o
	public Object removeMax();                   -> remove and return object with maximum key
    public double maxKey();						 -> return maximum key present in pq
	public Object maxElement();                  -> return object associated with max key (do not remove)
	public int size();							 -> return # items in pq
	public boolean isEmpty();                    -> check if heap is empty.

	public MaximizingBinaryHeapArrayPQ( int capacity );  -> create new pq with given capacity
 */

/** overall implementation idea: store negate keys when storing them,
     negate them again if the keys are ever looked at. Otherwise, use
     minimizing heap implementation. */
public class MaximizingBinaryHeapArrayPQ extends BinaryHeapArrayPQ
    {
        /**
         * Construct the binary heap.
         */
        public MaximizingBinaryHeapArrayPQ( )
        {
            super();
        }

        /**
         * Construct the binary heap.
         * @param capacity the capacity of the binary heap.
         */
        public MaximizingBinaryHeapArrayPQ( int capacity )
        {
           super(capacity);
        }


/**
 * associate key k with object o
 */
		public void insertItem(double k, Object o) throws Overflow{
			super.insertItem(-k,o);
		}

/**
 * remove and return object with maximum key
 */
		public Object removeMax() {
			return super.removeMin();
		}

/**
 * return maximum key present in pq
 */
		public double maxKey() {
			return -super.minKey();
		}

/**
 *return object associated with max key (do not remove)
 */
		public Object maxElement() {
			return super.minElement();
		}

	// Test program
        public static void main( String [ ] args )
        {
            int numItems = 20;
            MaximizingBinaryHeapArrayPQ h = new MaximizingBinaryHeapArrayPQ( 20 );
	    java.util.Random r = new java.util.Random();
	    int i, temp;
	    i =0;
            try
		{
		    for( i = 0; i < numItems; i++ ) {
			temp = r.nextInt(1000);
			h.insertItem( temp, "num" + i  );
			System.out.println("Inserted num" + i + " w/ key " + temp);
		    }
		    for( i = 0; i < numItems; i++ ) {
			System.out.println("maxKey is " + h.maxKey());
			System.out.println("maxElement is " + h.maxElement());
			System.out.println("removeMax returned " + h.removeMax());
		    }
		}
            catch( Overflow e )
		{ System.out.println( "Overflow! " + i  ); }
        }
    }
