/* This is a simple extension of the UnsortedArrayPQ to make
   it be a maximizing heap instead of a minimizing heap.
 * Created by Ed Hong, 2/17/05
 *
 * The relevant implemented calls are
 	public void insertItem(double k, Object o);  -> associate key k with object o
	public Object removeMax();                   -> remove and return object with maximum key
	public int size();							 -> return # items in pq
	public boolean isEmpty();                    -> check if heap is empty.

 	public Locator insert(double k, Object o);  -> associate key k with object o and return a Locator to it.
	public void replaceKey(Locator l, double k); -> Update key of item with locator l.
	public MaximizingUnsortedArrayPQ(  );  -> create new pq
 */



/** overall implementation idea: store negate keys when storing them,
     negate them again if the keys are ever looked at. Otherwise, use
     minimizing heap implementation. */
public class MaximizingUnsortedArrayPQ extends UnsortedArrayPQ
    {
        /**
         * Construct the PQ
         */
        public MaximizingUnsortedArrayPQ( )
        {
            super();
        }


/**
 * associate key k with object o
 */
		public void insertItem(double k, Object o) {
			insert(k,o);
		}

/**
 * remove and return object with maximum key
 */
		public Object removeMax() {
			return super.removeMin();
		}

    public Locator insert(double key, Object o) {
	return super.insert(-key,o);
    }

    public void replaceKey(Locator l, double newkey) {
	super.replaceKey(l,-newkey);
    }
	// Test program
        public static void main( String [ ] args )
        {
            int numItems = 10;
            MaximizingUnsortedArrayPQ h = new MaximizingUnsortedArrayPQ( );
	    java.util.Random r = new java.util.Random();
	    int i, temp;
	    Locator k;
	    i =0;
			temp = r.nextInt(1000);
			k= h.insert( temp, "num" + i  );
			System.out.println("Inserted num" + i + " w/ key " + temp);
		{
		    for( i = 1; i < numItems; i++ ) {
			temp = r.nextInt(1000);
			h.insertItem( temp, "num" + i  );
			System.out.println("Inserted num" + i + " w/ key " + temp);
		    }
		    System.out.println("removeMax returned " + h.removeMax());
		   		    System.out.println("Replacing item " + k.element() + " with key 2000.0");
		   		    h.replaceKey(k,2000.0);
		    for( i = 1; i < numItems; i++ ) {
		    System.out.println("removeMax returned " + h.removeMax());
		    }
		}
        }
    }
