/*
 * Written by Ed Hong UWT Feb. 17, 2003.
 * Modified by Donald Chinn May 14, 2003.
 * Modified by Ed Hong Nov 23, 2004.
 */

import java.util.*;

/**
 * A class that represents a graph.
 * 
 * @author edhong
 * @version 0.0
 */
public class SimpleGraph {

    LinkedList vertexList;
    LinkedList edgeList;

    // Constructor
    public SimpleGraph() {
        this.vertexList = new LinkedList();
        this.edgeList = new LinkedList();
    }
    
    /**
     * Return the vertex list of this graph.
     * @returns  vertex list of this graph
     */
    public Iterator vertices() {
        return vertexList.iterator();
    }

    /**
     * Return the edge list of this graph.
     * @returns  edge list of this graph
     */
    public Iterator edges() {
        return edgeList.iterator();
    }

    /**
     * Given a vertex, return an iterator to the edge list of that vertex
     * @param v  a vertex
     * @returns  an iterator to the edge list of that vertex
     */
    public Iterator incidentEdges(Vertex v) {
        return v.incidentEdgeList.iterator();
    }

    /**
     * Return an arbitrary vertex of this graph
     * @returns  some vertex of this graph
     */
    public Vertex aVertex() {
        if (vertexList.size() > 0)
            return (Vertex) vertexList.getFirst();
        else
            return null;
    }

    /**
     * Add a vertex to this graph.
     * @param o  an object to be associated with the new vertex
     * @returns  the new vertex
     */
    public Vertex insertVertex(Object o) {
        Vertex v;
        v = new Vertex(o);
        vertexList.addLast(v);
        return v;
    }

    /**
     * Add an edge to this graph.
     * @param v  the first endpoint of the edge
     * @param w  the second endpoint of the edge
     * @param o  an object to be associated with the new edge
     * @returns  the new edge
     */
    public Edge insertEdge(Vertex v, Vertex w, Object o) {
        Edge e;
        e = new Edge(v, w, o);
        edgeList.addLast(e);
        v.incidentEdgeList.addLast(e);
        w.incidentEdgeList.addLast(e);
        return e;
    }

    /**
     * Given a vertex and an edge, if the vertex is one of the endpoints
     * of the edge, return the other endpoint of the edge.  Otherwise,
     * return null.
     * @param v  a vertex
     * @param e  an edge
     * @returns  the other endpoint of the edge (or null, if v is not an endpoint of e)
     */
    public Vertex opposite(Vertex v, Edge e) {
        Vertex w;
        
        if (e.v1 == v) {
            w= e.v2;
        }
        else if (e.v2 == v) {
            w = e.v1;
        }
        else
            w = null;
        
        return w;
    }
    
    /**
     * Return the number of vertices in this graph.
     * @returns  the number of vertices
     */
    public int numVertices() {
        return vertexList.size();
    }

    /**
     * Return the number of edges in this graph.
     * @returns  the number of edges
     */
    public int numEdges() {
        return edgeList.size();
    }

    /**
     * Creates a string representation of this graph.
     * @returns  string representing this graph
     */
    public String toString() {
        return "Vertices: " + vertexList.toString() + 
			"\nEdges: " + edgeList.toString();
    }

    /**
     * Code to test the correctness of the SimpleGraph methods.
     */
    public static void main(String[] args) {
        // create graph a----b-----c,
        //                X     Y
        // X and Y are objects stored at edges. .

        // All Objects stored will be strings.

        SimpleGraph G = new SimpleGraph();
        Vertex v, w, a, b, c;
        Edge e, x, y;
        v = G.insertVertex("a");
        a = v;
        w = G.insertVertex("b");
        b = w;
        e = G.insertEdge(v,w,"X");
        x = e;
        v = G.insertVertex("c");
        c = v;
        e = G.insertEdge(w,v,"Y");
        y = e;

	    System.out.println("G = " + G);

        Iterator i;

        System.out.println("Iterating through vertices...");
        for (i= G.vertices(); i.hasNext(); ) {
            v = (Vertex) i.next();
            System.out.println("found vertex " + v.element());
        }

        System.out.println("Iterating through adjacency lists...");
        for (i= G.vertices(); i.hasNext(); ) {
            v = (Vertex) i.next();
            System.out.println("Vertex "+v.element());
            Iterator j;
            
            for (j = G.incidentEdges(v); j.hasNext();) {
                e = (Edge) j.next();
                System.out.println("  found edge " + e.element());
            }
        }

        System.out.println("Testing opposite...");
        System.out.println("aXbYc is ");
        System.out.println(a);
        System.out.println(x);
        System.out.println(b);
        System.out.println(y);
        System.out.println(c);

        System.out.println("opposite(a,x) is " + G.opposite(a,x));
        System.out.println("opposite(a,y) is " + G.opposite(a,y));
        System.out.println("opposite(b,x) is " + G.opposite(b,x));
        System.out.println("opposite(b,y) is " + G.opposite(b,y));
        System.out.println("opposite(c,x) is " + G.opposite(c,x));
        System.out.println("opposite(c,y) is " + G.opposite(c,y));

    }
}




