/*
Written by Ed Hong
UWT
Jan. 22, 2003
modified June 8, 2003
modified May 26, 2004
Modified 2/14/05
 * This implements a minimizing Unsorted-Array based
 * Priority Queue, where the key is always a double. This is
 * only a partial implementation and is not complete. It also contains a
 * partial implementation of the extend PQ ADT set, including  replaceKey(),
 * and locators. Also note that this priority does not
 * use comparators. This implementation may not handle errors well.
 * The relevant implemented calls are
 	public void insertItem(double k, Object o);  -> associate key k with object o
	public Object removeMin();                   -> remove and return object with minimum key
	public int size();							 -> return # items in pq
	public boolean isEmpty();                    -> check if heap is empty.

 	public Locator insert(double k, Object o);  -> associate key k with object o and return a Locator to it.
	public void replaceKey(Locator l, double k); -> Update key of item with locator l.
	public UnsortedArrayPQ( );                    -> create new pq */

import java.util.*;


public class UnsortedArrayPQ {

   // internal class for containing
    // pq items, including (key,object,and locator).
    // no methods beside constructor.
  private class pqitem {
      public double key;
      public Object element;
      public uapqLoc loc;
      public pqitem(double k, Object o) {
		key = k;
		element = o;
		loc = new uapqLoc(this); // initialize locator to point to this item
      }
  }

  private class uapqLoc implements Locator {
      private pqitem locatordat; // reference to item inside pq
      // constructor..
      private uapqLoc(pqitem k) {
		locatordat = k;
      }
      public double key() {
	  /* returns key (priority) associated to referenced data */
		return locatordat.key;
      }
      public Object element() {
	  /* returns Object associated to referenced data */
		return locatordat.element;
      }
  }

    private ArrayList pqdata;

    /** create a new pq */
	public UnsortedArrayPQ() {
		pqdata = new ArrayList();
    }

/**
 * associate key k with object o
 */
	public void insertItem(double k, Object o) {
	    insert(k,o);
	}

/**
 * associate key k with object o, return a locator to it.
 */
    public Locator insert(double key, Object o) {
		pqitem temp;
		temp = new pqitem(key, o);
		pqdata.add(temp);
		return temp.loc;
    }

/**
 * remove and return object with minimum key
 */
    public Object removeMin() {
		int ind; // stores index of element with smallest key
		double smallkey; // stores smallest key
		pqitem temp;

		if (pqdata.size()==0) return null;

		ind =0;
		temp = (pqitem) pqdata.get(0);
		smallkey = temp.key;
	//	    System.out.println("PQDAT: item " + temp + " (" + 0 + ") is (" +
	//	       temp.key + ", " +
	//	       temp.element + ")");
	// search for item with smallest key.
		for (int i=1; i < pqdata.size(); i++) {
			temp = (pqitem) pqdata.get(i);
	    //	    System.out.println("PQDAT: item " + temp + " (" + i + ") is (" +
	    //	       temp.key + ", " +
	    //	       temp.element + ")");
			if (temp.key < smallkey) {
			smallkey = temp.key;
			ind = i;
			}
		}
		temp = (pqitem) pqdata.get(ind);
		pqdata.remove(ind);
		temp.loc.locatordat = null;
		return temp.element;
    }

/**
  *  Update key of item with locator l.
  */
    public void replaceKey(Locator l, double newkey) {
		uapqLoc ul = (uapqLoc) l;
		if (ul.locatordat==null) {
			System.out.println("Error in replaceKey(): locator refers to object not in PQ! Ignoring call!");
			return;
		}
		ul.locatordat.key = newkey;
	//	System.out.println("ReplaceKey:  newkey " + newkey +
	//	   "; locaterdat = " + l.locatordat + ";");
	//  System.out.println("RPQDAT: item " + l.locatordat + " is (" +
	//	       l.locatordat.key + ", " +
	//	       l.locatordat.element + ")");
    }

    /**
         * Test if the priority queue is logically empty.
         * @return true if empty, false otherwise.
         */
    public boolean isEmpty() {
		return (pqdata.size() == 0);
    }


/**
 * return # items in pq
 */
	public int size() {
	  return pqdata.size();
	}

	// Test program
        public static void main( String [ ] args )
        {
            int numItems = 10;
            UnsortedArrayPQ h = new UnsortedArrayPQ( );
	    java.util.Random r = new java.util.Random();
	    int i, temp;
	    Locator k;
	    i =0;
			temp = r.nextInt(1000);
			k= h.insert( temp, "num" + i  );
			System.out.println("Inserted num" + i + " w/ key " + temp);
		{
		    for( i = 1; i < numItems; i++ ) {
			temp = r.nextInt(1000);
			h.insertItem( temp, "num" + i  );
			System.out.println("Inserted num" + i + " w/ key " + temp);
		    }
		    System.out.println("removeMin returned " + h.removeMin());
		    System.out.println("Replacing item " + k.element() + " with key 2000.0");
		    h.replaceKey(k,2000.0);
		    for( i = 1; i < numItems; i++ ) {
		    System.out.println("removeMin returned " + h.removeMin());
		    }
		}
        }

}

